######################################################################
#                      Inicio de corrector_TC.sh                     #
######################################################################

export RUTA_CORREGIDOS=""

crea_dir_correg

check_informe_det
ret_corrector=$?

if [[ $ret_corrector == 1 ]]
then
	log "INICIALIZANDO CORRECTOR" "TI"
elif [[ $ret_corrector == 0 ]]
then
	log "ABANDONANDO CORRECTOR" "TI"
	return
else
	log "017 Error de retorno de la función check_informe_det" "E"
fi

awk -b -v informe_det=$INFORME_DET -v fich_log=$LOG -v ruta_corregidos=$RUTA_CORREGIDOS -v ruta_ficheros=$RUTA_FICHEROS '
######################################################################
#              Nombre función: genera_array_arreglos                 #
#     Descripción: genera un array con las expresiones de matcheo    #
#                      y sustitución de cada subregla                #
######################################################################
function genera_array_arreglos()
{	
	##################################################################
	#                            REGLA 1                             #
	##################################################################
	arreglos["0101"][0]["regexp"]=""
	arreglos["0101"][0]["replac"]=""
	arreglos["0101"][1]["regexp"]=""
	arreglos["0101"][1]["replac"]=""
	
	arreglos["0102"][0]["regexp"]=""
	arreglos["0102"][0]["replac"]=""
	
	##################################################################
	#                            REGLA 2                             #
	##################################################################
	arreglos["0201"][0]["regexp"]=""
	arreglos["0201"][0]["replac"]=""
	arreglos["0201"][1]["regexp"]=""
	arreglos["0201"][1]["replac"]=""
	
}
######################################################################
#                   Nombre función: log_awk                          #
#     Descripción: imprime la cadena que recibe en el fichero        #
#       correspondiente según el nivel del texto a imprimir          #
#     Argumentos: texto --> cadena a imprimir                        #
#                 nivel --> Nivel del texto                          #
#                      I --> Información en log                      #
#                      TI -> Información en terminal e log           #
######################################################################
function log_awk(texto,nivel)
{
	if (nivel == "I") {
		# Pinta en log.txt
		print "[INFO] ---- " texto >> fich_log
	} else if (nivel == "TI") {
		# Pinta en log.txt y en el terminal
		print "[INFO] ---- " texto >> fich_log
		print "[INFO] ---- " texto
	} else if (nivel == "E") {
		# Pinta en log.txt y en el terminal
		print "[ERROR]" texto >> fich_log
		print "[ERROR]" texto
		exit 1
	}
}

######################################################################
#              Nombre función: carga_informe_detector                #
#     Descripción: carga el informe del detector en un array         #
######################################################################
function carga_informe_detector()
{
	getline error < informe_det
	fichero_anterior=""
	linea_anterior=""
	
	while (getline error < informe_det) {
		split(error,err_campos,";")
		fichero=err_campos[1]
		linea=err_campos[2]
		subregla=err_campos[4]
		contador_arreglos=0
		
		if (fichero in array_errores && linea in array_errores[fichero]) {
			contador_arreglos=length(array_errores[fichero][linea])
		}
		fichero_anterior=fichero
		linea_anterior=linea

		array_errores[fichero][linea][contador_arreglos]=subregla
	}
	close(informe_det)
}
######################################################################
#                 Nombre función: aplica_reglas                      #
#     Descripción: modifica la línea del fichero aplicando la        #
#                      subregla correspondiente                      #
#     Argumentos: linea  --> línea a tratar                          #
#                 fichero  --> fichero al que pertenece la linea     #
#                 num_linea  --> número de línea de línea            #
######################################################################
function aplica_reglas(linea, fichero, num_linea)
{
	linea_fix[1]=linea
	
	if(fichero in array_errores && num_linea in array_errores[fichero]) {
		for(num_subreglas in array_errores[fichero][num_linea]) {
			subregla = array_errores[fichero][num_linea][num_subreglas]
			if (subregla in arreglos){
			
				for(indice_subregla in arreglos[subregla]) {
					linea_fix[1] = gensub(arreglos[subregla][indice_subregla]["regexp"], arreglos[subregla][indice_subregla]["replac"], "g", linea_fix[1])
				}
			} else {
				log_awk("Subregla " subregla " no definida en corrector","TI")
			}
			n_correcciones++
		}
		split(linea_fix[1],linea_fix,"\n")
	}
	
	for (ind in linea_fix) {
		escribe_linea(linea_fix[ind], fichero)
		delete linea_fix[ind]
	}
}

######################################################################
#                 Nombre función: escribe_linea                      #
#     Descripción: escribe la línea en el fichero correspondiente    #
#     Argumentos: linea  --> línea a escribir                        #
#                 fichero  --> fichero en el que escribir            #
#                 identacion --> número de espacios de identación    #
######################################################################
function escribe_linea(linea, fichero)
{	
	sub(ruta_ficheros"\\/","",fichero)
	fichero=ruta_corregidos"/"fichero
	
	if (linea !~ /^\s*_NO-IMPRIMIR_$/) {
		if (linea ~ /_ELIM-RETORNO_/) {
			sub(/\s*_ELIM-RETORNO_/," ",linea)
			printf "%s" , linea >> fichero
		} else {
			print linea >> fichero
		}
	}
}

BEGIN {
	n_correcciones=0
	
	carga_informe_detector()
	genera_array_arreglos()
}{	
	aplica_reglas($0, FILENAME, FNR)
}

END {
	log_awk("Se han realizado un total de " n_correcciones " correcciones","TI")
}' $WORKSHOP


ret_corrector=$?
if [[ $ret_corrector == 0 ]]
then
	log "CORRECTOR FINALIZÓ SIN INCIDENCIAS" "TI"
	if [[ $SOBRESCRIBIR == "SI" ]]
	then
		rm -rf $RUTA_FICHEROS
		log "${RUTA_FICHEROS} eliminado" "I"
		mv $RUTA_CORREGIDOS $RUTA_FICHEROS
		log "${RUTA_CORREGIDOS} es ahora ${RUTA_FICHEROS}" "I"
		RUTA_CORREGIDOS=$RUTA_FICHEROS
	fi
else
	log "018 CORRECTOR NO FINALIZÓ CORRECTAMENTE" "E"
fi
