######################################################################
#                      Inicio de corrector_TC.sh                     #
######################################################################

export RUTA_CORREGIDOS=""

crea_dir_correg

check_informe_det
ret_corrector=$?

if [[ $ret_corrector == 1 ]]
then
	log "INICIALIZANDO CORRECTOR" "TI"
elif [[ $ret_corrector == 0 ]]
then
	log "ABANDONANDO CORRECTOR" "TI"
	return
else
	log "017 Error de retorno de la función check_informe_det" "E"
fi

awk -b -v informe_det=$INFORME_DET -v fich_log=$LOG -v ruta_corregidos=$RUTA_CORREGIDOS -v ruta_ficheros=$RUTA_FICHEROS -v regla1="${regla[1]}" '
######################################################################
#              Nombre función: genera_array_arreglos                 #
#     Descripción: genera un array con las expresiones de matcheo    #
#                      y sustitución de cada subregla                #
######################################################################
function genera_array_arreglos()
{
	##################################################################
	#                Expresiones regulares constantes                #
	##################################################################
	
	# Encaja con paréntesis que tengan dentro de ellos paréntesis emparejados
	parent_par="\\s*\\([^\\(\\)]*(\\([^\\(\\)]*\\)[^\\(\\)]*)*\\)"
	parent_par="\\s*\\([^\\)]*(\\([^\\(\\)]*\\)[^\\(]*)*\\)"
	
	tipos_var="^\\s*(const\\s+|signed\\s+|unsigned\\s+|char\\s+|short\\s+|long\\s+|int\\s+|float\\s+|double\\s+|auto\\s+|bool\\s+)"
	
	##################################################################
	#                            REGLA 1                             #
	##################################################################
	arreglos["0101"][0]["regexp"]="\\s*({)"
	arreglos["0101"][0]["replac"]="\n\\1"
	arreglos["0101"][1]["regexp"]="^\n"
	arreglos["0101"][1]["replac"]=""
	
	arreglos["0102"][0]["regexp"]="\\s*(})"
	arreglos["0102"][0]["replac"]="\n\\1"
	arreglos["0102"][1]["regexp"]="^\n"
	arreglos["0102"][1]["replac"]=""
	
	arreglos["0103"][0]["regexp"]="^\\s*"
	arreglos["0103"][0]["replac"]=""
	
	##################################################################
	#                            REGLA 2                             #
	##################################################################
	arreglos["0201"][0]["regexp"]="^\\s*(.*)"
	arreglos["0201"][0]["replac"]="\\1"
	arreglos["0201"][1]["regexp"]="\\t"
	arreglos["0201"][1]["replac"]="    "
	
	##################################################################
	#                            REGLA 3                             #
	##################################################################
	arreglos["0301"][0]["regexp"]="^\\s*"
	arreglos["0301"][0]["replac"]=""
	arreglos["0301"][1]["regexp"]="(\\)|else|do)(\\s*(\\/\\/.*|\\/\\*.*)?)$"
	arreglos["0301"][1]["replac"]="\\1 {\\2"
	
	
	arreglos["0302"][0]["regexp"]="^\\s*(.*)"
	arreglos["0302"][0]["replac"]="\\1\n}"
	
	arreglos["0303"][0]["regexp"]="^\\s*(else|while)"
	arreglos["0303"][0]["replac"]="} \\1"
	
	arreglos["0304"][0]["regexp"]="\\s*(.*)((if|for|while)"parent_par"|else(\\s*if"parent_par")?|\\sdo)\\s*"
	arreglos["0304"][0]["replac"]="\\1\\2 {\n"
	
	arreglos["0305"][0]["regexp"]="^\\s*"
	arreglos["0305"][0]["replac"]=""
	arreglos["0305"][1]["regexp"]="\\/\\/(.*)"
	arreglos["0305"][1]["replac"]="\\/\\*\\1\\*\\/"
	arreglos["0305"][2]["regexp"]="(.*)\\s*(\\/\\/|\\/\\*|)"
	arreglos["0305"][2]["replac"]="\\1_ELIM-RETORNO_\\2"
	
	##################################################################
	#                            REGLA 4                             #
	##################################################################
	arreglos["0401"][1]["regexp"]="^\\s*"
	arreglos["0401"][1]["replac"]=""
	arreglos["0401"][0]["regexp"]="(\\)|else|do|try|case.*:|default:)(\\s*){?(\\s*(\\/\\/.*|\\/\\*.*)?)$"
	arreglos["0401"][0]["replac"]="\\1 {\\2\\3"
	
	
	arreglos["0402"][0]["regexp"]="^\\s*}?\\s*(catch"parent_par"|while"parent_par"\\s*;|else(\\s*if"parent_par")?)"
	arreglos["0402"][0]["replac"]="} \\1"
	
	arreglos["0403"][0]["regexp"]="^\\s*"
	arreglos["0403"][0]["replac"]=""
	arreglos["0403"][1]["regexp"]="^\\s*{\\s*"
	arreglos["0403"][1]["replac"]=""
	arreglos["0403"][2]["regexp"]="^\\s*$"
	arreglos["0403"][2]["replac"]="_NO-IMPRIMIR_"
	
	arreglos["0404"][0]["regexp"]="^\\s*"
	arreglos["0404"][0]["replac"]=""
	arreglos["0404"][1]["regexp"]="\\s*}\\s*$"
	arreglos["0404"][1]["replac"]=""
	arreglos["0404"][2]["regexp"]="^\\s*$"
	arreglos["0404"][2]["replac"]="_NO-IMPRIMIR_"
	
	arreglos["0405"][0]["regexp"]="^\\s*(}\\s*)?((if|for|while|catch|switch)"parent_par"|else(\\s*if"parent_par")?|do|try|case.*:|default:)\\s*{\\s*"
	arreglos["0405"][0]["replac"]="\\1\\2 {\n"
	arreglos["0405"][1]["regexp"]=";\\s*}"
	arreglos["0405"][1]["replac"]=";\n}"
	
	##################################################################
	#                            REGLA 5                             #
	##################################################################
	arreglos["0501"][0]["regexp"]="^\\s*"
	arreglos["0501"][0]["replac"]=""
	arreglos["0501"][1]["regexp"]="\\s*{"
	arreglos["0501"][1]["replac"]="\n{"
	arreglos["0501"][2]["regexp"]="{\\s*(.+)"
	arreglos["0501"][2]["replac"]="{\n\\1"
	
	##################################################################
	#                            REGLA 6                             #
	##################################################################
	arreglos["0601"][0]["regexp"]="^\\s*"
	arreglos["0601"][0]["replac"]=""
	arreglos["0601"][1]["regexp"]=",\\s*$"
	arreglos["0601"][1]["replac"]=";"
	arreglos["0601"][2]["regexp"]="\\s*,\\s*"
	arreglos["0601"][2]["replac"]=";\n"
	arreglos["0601"][3]["regexp"]="^\\s*;\n\\s*"
	arreglos["0601"][3]["replac"]=""
	
	arreglos["0602"][0]["regexp"]="^\\s*"
	arreglos["0602"][0]["replac"]=""
	arreglos["0602"][1]["regexp"]=",\\s*$"
	arreglos["0602"][1]["replac"]=";"
	arreglos["0602"][2]["regexp"]="\\s*,\\s*"
	arreglos["0602"][2]["replac"]=";\n"
	arreglos["0602"][3]["regexp"]="^\\s*;\n\\s*"
	arreglos["0602"][3]["replac"]=""
	
	arreglos["0603"][0]["regexp"]="(.*)"
	arreglos["0603"][0]["replac"]=",\\1"
	
	arreglos["0604"][0]["regexp"]="(.*)"
	arreglos["0604"][0]["replac"]="\\1;"
	
	##################################################################
	#                            REGLA 7                             #
	##################################################################
	arreglos["0701"][0]["regexp"]="^\\s*"
	arreglos["0701"][0]["replac"]=""
	arreglos["0701"][1]["regexp"]="\\/(\\/+)\\s*"
	arreglos["0701"][1]["replac"]="\\/\\1 "
	
	arreglos["0702"][0]["regexp"]="^\\s*(.*)\\/(\\*)+"
	arreglos["0702"][0]["replac"]="\\1\n\\/\\*"
	
	arreglos["0703"][0]["regexp"]="^\\s*(.*)\\/(\\*)+\\s*(.*)"
	arreglos["0703"][0]["replac"]="\\1\n\\/\\*\n \\* \\3"
	
	arreglos["0704"][0]["regexp"]="^\\s*(\\*)+\\/\\s*(.*)"
	arreglos["0704"][0]["replac"]=" \\*\\/\n\\2"
	
	arreglos["0705"][0]["regexp"]="(\\*)+\\/"
	arreglos["0705"][0]["replac"]="\\*\\/"
	arreglos["0705"][1]["regexp"]="^\\s*(.*)\\s*(\\*)+\\/(.*)$"
	arreglos["0705"][1]["replac"]=" \\* \\1\n \\*\\/\n\\3"
	
	arreglos["0706"][0]["regexp"]="^\\s*(\\**\\s*)(.*)\\s*"
	arreglos["0706"][0]["replac"]=" \\* \\2"
	
	##################################################################
	#                            REGLA 8                             #
	##################################################################
	arreglos["0801"][0]["regexp"]="^\\s*"
	arreglos["0801"][0]["replac"]=""
	arreglos["0801"][1]["regexp"]="\\s*(\\|\\||&&)\\s*"
	arreglos["0801"][1]["replac"]=" \\1 "
}
######################################################################
#                   Nombre función: log_awk                          #
#     Descripción: imprime la cadena que recibe en el fichero        #
#       correspondiente según el nivel del texto a imprimir          #
#     Argumentos: texto --> cadena a imprimir                        #
#                 nivel --> Nivel del texto                          #
#                      I --> Información en log                      #
#                      TI -> Información en terminal e log           #
######################################################################
function log_awk(texto,nivel)
{
	if (nivel == "I") {
		# Pinta en log.txt
		print "[INFO] ---- " texto >> fich_log
	} else if (nivel == "TI") {
		# Pinta en log.txt y en el terminal
		print "[INFO] ---- " texto >> fich_log
		print "[INFO] ---- " texto
	} else if (nivel == "E") {
		# Pinta en log.txt y en el terminal
		print "[ERROR]" texto >> fich_log
		print "[ERROR]" texto
		exit 1
	}
}
######################################################################
#                   Nombre función: mi_getline                       #
#     Descripción: devuelve la cadena correspondiente al fichero     #
#                      y linea dados                                 #
#     Argumentos: fichero --> nombre del fichero                     #
#                 n_linea --> nº de línea                            #
#     Return: línea leída                                            #
######################################################################
function mi_getline(fichero, n_linea)
{
        cont=0
        do {
        	cont++
        	getline linea_getline < fichero
        } while (cont<n_linea)
		close(fichero)
        return linea_getline
}

######################################################################
#                 Nombre función: cuenta_ocurrencias                 #
#     Descripción: devuelve el nº de ocurrencias que encajan con     #
#                      regexp                                        #
#     Argumentos: regexp --> expresión regular                       #
#                 linea  --> línea a tratar                          #
#     Return: número de ocurrencias                                  #
######################################################################
function cuenta_ocurrencias(regexp,linea)
{
	linea_aux=linea
	for (iterador=0; linea_aux ~ regexp; iterador++) {
		sub(regexp,"",linea_aux)
	}
	return iterador
}

######################################################################
#              Nombre función: carga_informe_detector                #
#     Descripción: carga el informe del detector en un array         #
######################################################################
function carga_informe_detector()
{
	getline error < informe_det
	fichero_anterior=""
	linea_anterior=""
	
	while (getline error < informe_det) {
		split(error,err_campos,";")
		fichero=err_campos[1]
		linea=err_campos[2]
		subregla=err_campos[4]
		contador_arreglos=0
		
		if (fichero in array_errores && linea in array_errores[fichero]) {
			contador_arreglos=length(array_errores[fichero][linea])
		}
		fichero_anterior=fichero
		linea_anterior=linea

		array_errores[fichero][linea][contador_arreglos]=subregla
	}
	close(informe_det)
}
######################################################################
#                 Nombre función: aplica_reglas                      #
#     Descripción: modifica la línea del fichero aplicando la        #
#                      subregla correspondiente                      #
#     Argumentos: linea  --> línea a tratar                          #
#                 fichero  --> fichero al que pertenece la linea     #
#                 num_linea  --> número de línea de línea            #
######################################################################
function aplica_reglas(linea, fichero, num_linea)
{
	linea_fix[1]=linea
	borra_ident=1
	no_identar=0
	
	if(fichero in array_errores && num_linea in array_errores[fichero]) {
		for(num_subreglas in array_errores[fichero][num_linea]) {
			subregla = array_errores[fichero][num_linea][num_subreglas]
			if (subregla in arreglos){
			
				for(indice_subregla in arreglos[subregla]) {
					linea_fix[1] = gensub(arreglos[subregla][indice_subregla]["regexp"], arreglos[subregla][indice_subregla]["replac"], "g", linea_fix[1])
				}
				if (subregla ~ /0103/) {
					no_identar=1
				}
				borra_ident=0
			} else {
				log_awk("Subregla " subregla " no definida en corrector","TI")
			}
			n_correcciones++
		}
		split(linea_fix[1],linea_fix,"\n")
	}
	
	for (ind in linea_fix) {
		
		# Si la regla de identación está activa
		if (regla1) {
			if (no_identar) {
				calc_ident(linea_fix[ind],borra_ident)
			} else {
				linea_fix[ind]=calc_ident(linea_fix[ind],borra_ident)
			}
		}
		
		escribe_linea(linea_fix[ind], fichero)
		delete linea_fix[ind]
	}
}
######################################################################
#                 Nombre función: calc_ident                         #
#     Descripción: calcula la identación que le corresponde a línea  #
#     Argumentos: linea  --> línea a tratar                          #
#                 borra_ident --> si es 1 eliminamos la identación   #
#                     previa, si es 0, la mantenemos                 #
#     Return: línea identada correctamente                           #
######################################################################
function calc_ident(linea,borra_ident) 
{
	ident_actual=ident_sig
	linea_sig=mi_getline(FILENAME,FNR+1)
	
	if (FNR == 1) {
		ident_actual=0
		ident_sig=0
		n_llaves_class=0
		flag_class=0
		ident_switch=0
		ident_class=0
		n_llaves_switch=0
		n_ocurr=0
		multilinea_redir=0
		multilinea_operac=0
		primera_linea=0
	}
	
	linea_mod=elimina_coment(linea)
	n_abre_llave=cuenta_ocurrencias("{",linea_mod)
	n_cierra_llave=cuenta_ocurrencias("}",linea_mod)
	
	if (linea_mod ~ /^\s*switch\s*\(.*\)*\s*({|$)/) {
		# ident_switch es la ident +1 para poder usar la variable también para control
		ident_switch=1
		n_llaves_switch+=n_abre_llave-n_cierra_llave
	} else if (ident_switch) {
		n_llaves_switch+=n_abre_llave-n_cierra_llave
		if (n_llaves_switch == 0) {
			ident_switch=0
			flag_switch=0
			ident_actual-=4
			ident_sig-=4

		} else {
			if (linea_mod ~ /^\s*case\s+.*:/ || linea_mod ~ /^\s*default\s*:/) {
				if ( flag_switch == 0 ) {
					flag_switch=1
					ident_sig+=4
				} else {
					ident_actual-=4
				}
			}
			
		}
		
	}
	
	if ( (linea_mod ~ /(std::)?(cout|cin)\s*(<<|>>|$)/) || multilinea_redir) {
		if (!multilinea_redir) {
			multilinea_redir=1
			ident_sig+=8
		}
		if (linea_mod ~ /;\s*$/) {
			ident_sig-=8
			multilinea_redir=0
		}
	} else if ((linea_mod ~ /[^=]=[^=]/ || linea_sig  ~ /^\s*=[^=]/) && ( linea_mod ~ /(-|+|*[^\/]|=|\/[^\*\/])\s*$/ || linea_sig ~ /^\s*(-|+|*[^\/]|=|\/[^\*\/])/ ) && (linea_mod !~ /{\s*$/ && linea_mod !~ /;\s*$/) || multilinea_operac) {
		if (!multilinea_operac) {
			multilinea_operac=1
			ident_sig+=8
		}
		if (linea_mod ~ /;\s*$/) {
			ident_sig-=8
			multilinea_operac=0
		}
	} else {
		if (linea_mod ~ /{([0-9]+,)+}/) {
			ident_multilinea(linea_mod,"{","}")
		} else {
			if (linea_mod ~ /{/ && linea_mod !~ /{.*}/) {
				ident_sig+=4*n_abre_llave
			}
			
			if (linea_mod ~ /}/ && linea_mod !~ /{.*}/) {
				ident_actual-=4*n_cierra_llave
				ident_sig-=4*n_cierra_llave
			}
			ident_multilinea(linea_mod,"\\(","\\)")
		}
	}
	
	
	
	if (linea_mod ~ /^\s*class\s+/ && linea_mod !~ /;\s*$/) {
		# ident_class es la ident +1 para poder usar la variable también para control
		ident_class=1
		n_llaves_class+=n_abre_llave-n_cierra_llave
	} else if (ident_class) {
		n_llaves_class+=n_abre_llave-n_cierra_llave
		if (n_llaves_class == 0) {
			ident_actual-=4
			ident_sig-=4
			ident_class=0
			flag_class=0
		} else {
			if (linea_mod ~ /^\s*(public|private|protected)\s*:/) {
				if ( flag_class == 0 ) {
					flag_class=1
					ident_sig+=4
				} else {
					ident_actual-=4
				}
			}
			
		}
		
	}

	espacios=""
	if (ident_actual<0) {
		log_awk(" 021 Error con la identación del fichero "FILENAME" en el corrector","E")

	}
	for (i=0; i<ident_actual; i++) {
		espacios=(espacios " ")
	}
	
	if (borra_ident == 0) {
		sub(/^/,espacios,linea)
	} else {
		sub(/^\s*/,espacios,linea)
	}
	
	linea_ant=linea
	
	return linea
}
######################################################################
#                 Nombre función: elimina_coment                     #
#     Descripción: elimina los comentarios de línea                  #
#     Argumentos: linea  --> línea a tratar                          #
#     Return: línea modificada, ahora sin comentarios                #
######################################################################
function elimina_coment(linea)
{
	if (linea ~ /\".+\"/) {
		gsub(/\"[^\"]+\"/,"",linea)#"
	}
	
	if (linea ~ /\/\//) {
		sub(/\s*\/\/.*/,"",linea)
	}
	
	
	if (linea ~ /\/\*/ || coment_rang) {
		if (linea ~ /\/\*/ && linea ~ /\*\//) {
			sub(/\/\*.*\*\//,"",linea)
			coment_rang=0
		} else if (linea ~ /\/\*/) {
			sub(/\/\*.*/,"",linea)
			coment_rang=1
		} else if (linea ~ /\*\//) {
			sub(/.*\*\//,"",linea)
			coment_rang=0	
		} else {
			sub(/^.*$/,"",linea)
		}
	}
	
	return linea
}
######################################################################
#                 Nombre función: ident_multilinea                   #
#     Descripción: calcula la identación de una sentencia en varias  #
#                      líneas                                        #
#     Argumentos: linea  --> línea a tratar                          #
#                 exp_apert  --> exp regular del carácter apertura   #
#                 exp_cierre --> exp regular del carácter cierre     #
######################################################################
function ident_multilinea(linea,exp_apert,exp_cierre)
{
	dif_ocurr=cuenta_ocurrencias(exp_apert,linea)-cuenta_ocurrencias(exp_cierre,linea)
	n_ocurr=n_ocurr+dif_ocurr
	
	if (linea ~ /exp_apert/ || linea ~ /exp_cierre/ || n_ocurr || primera_linea ) {
		if (n_ocurr) {
			if (primera_linea == 0 ) {
				primera_linea++
				ident_sig+=8
			}
		} else if (dif_ocurr) {
			ident_sig-=8
			primera_linea=0
		}
		if (linea ~ /[{;]\s*$/) {
			if (primera_linea==0) {
				primera_linea++
				ident_sig+=8
			}
			if (linea ~ /[{;]/ && n_ocurr == 0) {
				if (linea ~/^\s*{\s*$/) {
					ident_actual-=8
				}
				ident_sig-=8
				primera_linea=0
			}
		}
	}
}
######################################################################
#                 Nombre función: escribe_linea                      #
#     Descripción: escribe la línea en el fichero correspondiente    #
#     Argumentos: linea  --> línea a escribir                        #
#                 fichero  --> fichero en el que escribir            #
#                 identacion --> número de espacios de identación    #
######################################################################
function escribe_linea(linea, fichero)
{	
	sub(ruta_ficheros"\\/","",fichero)
	fichero=ruta_corregidos"/"fichero
	
	if (linea !~ /^\s*_NO-IMPRIMIR_$/) {
		if (linea ~ /_ELIM-RETORNO_/) {
			sub(/\s*_ELIM-RETORNO_/," ",linea)
			printf "%s" , linea >> fichero
		} else {
			print linea >> fichero
		}
	}
}

BEGIN {
	ident_actual=0
	ident_sig=0
	n_llaves_class=0
	flag_class=0
	ident_switch=0
	ident_class=0
	n_llaves_switch=0
	n_ocurr=0
	multilinea_redir=0
	multilinea_operac=0
	primera_linea=0
	n_correcciones=0
	
	carga_informe_detector()
	genera_array_arreglos()
	
	or_06=arreglos["0601"][2]["replac"]
}{
	n_matches=0
	linea_match=$0
	while (match(linea_match,tipos_var) != 0) {
		if (n_matches == 0) {
			tipo=""
		}
		tipo=tipo""gensub(tipos_var".*", "\\1",1,linea_match)
		sub(tipos_var,"",linea_match)
		n_matches++
	}
	
	arreglos["0601"][2]["replac"]=or_06 tipo
	arreglos["0602"][2]["replac"]=or_06 tipo
	
	aplica_reglas($0, FILENAME, FNR)
}

END {
	log_awk("Se han realizado un total de " n_correcciones " correcciones","TI")
}' $WORKSHOP


ret_corrector=$?
if [[ $ret_corrector == 0 ]]
then
	log "CORRECTOR FINALIZÓ SIN INCIDENCIAS" "TI"
	if [[ $SOBRESCRIBIR == "SI" ]]
	then
		rm -rf $RUTA_FICHEROS
		log "${RUTA_FICHEROS} eliminado" "I"
		mv $RUTA_CORREGIDOS $RUTA_FICHEROS
		log "${RUTA_CORREGIDOS} es ahora ${RUTA_FICHEROS}" "I"
		RUTA_CORREGIDOS=$RUTA_FICHEROS
	fi
else
	log "018 CORRECTOR NO FINALIZÓ CORRECTAMENTE" "E"
fi
