######################################################################
#                      Inicio de detector_TC.sh                      #
######################################################################


log "INICIALIZANDO DETECTOR" "TI"

echo "Fichero;Linea;Regla;Subregla" >> $INFORME_DET
echo "Fichero;Linea;Regla;Regla Reporte;Incidencia" >> $REPORTE

awk -b -v reporte=$REPORTE -v informe_det=$INFORME_DET -v fich_log=$LOG -v reglas="${regla[*]}" -v num_reg_rep=$NUM_REG_REP -v long_max=$LONG_MAX '
######################################################################
#                   Nombre función: log_awk                          #
#     Descripción: imprime la cadena que recibe en el fichero        #
#       correspondiente según el nivel del texto a imprimir          #
#     Argumentos: texto --> cadena a imprimir                        #
#                 nivel --> Nivel del texto                          #
#                      I --> Información en log                      #
#                      TI -> Información en terminal e log           #
#                      ID -> Arreglos en informe_detector.csv        #
#                      R --> Reporte generado por regla              #
######################################################################
function log_awk(texto,nivel)
{
	if (nivel == "I") {
		# Pinta en log_TC.txt
		print "[INFO] ---- " texto >> fich_log
	} else if (nivel == "TI") {
		# Pinta en log_TC.txt y en el terminal
		print "[INFO] ---- " texto >> fich_log
		print "[INFO] ---- " texto
	} else if (nivel == "E") {
		# Pinta en log.txt y en el terminal
		print "[ERROR]" texto >> fich_log
		print "[ERROR]" texto
		exit 1
	} else if (nivel == "D") {
		# Pinta en informe_detector.csv
		print texto >> informe_det
	} else if (nivel == "R") {
		# Pinta en reportes.csv
		if (texto ~ /^[^;]+;[^;]+;[^;]+;1;$/) {
			texto=texto "Variable no inicializada"
		} else if (texto ~ /^[^;]+;[^;]+;[^;]+;2;$/) {
			texto=texto "La linea sobrepasa el limite de longitud de " long_max
		} else if (texto ~ /^[^;]+;[^;]+;[^;]+;3;$/) {
			texto=texto "Funcion sin comentario explicativo"
		}
		print texto >> reporte
	}
}
######################################################################
#                 Nombre función: elimina_coment                     #
#     Descripción: elimina los comentarios de línea                  #
#     Argumentos: linea  --> línea a tratar                          #
#     Return: línea modificada, ahora sin comentarios                #
######################################################################
function elimina_coment(linea)
{
	if (linea ~ /\/\//) {
		sub(/\s*\/\/.*/,"",linea)
	}
	
	if (linea ~ /\".+\"/) {
		sub(/\".+\"/,"",linea)
	}
	
	if (linea ~ /\/\*/ || coment_rang) {
		if (linea ~ /\/\*/ && linea ~ /\*\//) {
			sub(/\/\*.*\*\//,"",linea)
			coment_rang=0
		} else if (linea ~ /\/\*/) {
			sub(/\/\*.*/,"",linea)
			coment_rang=1
		} else if (linea ~ /\*\//) {
			sub(/.*\*\//,"",linea)
			coment_rang=0	
		} else {
			sub(/^.*$/,"",linea)
		}
	}
	
	return linea
}
######################################################################
#                 Nombre función: cuenta_ocurrencias                 #
#     Descripción: devuelve el nº de ocurrencias que encajan con     #
#                      regexp                                        #
#     Argumentos: regexp --> expresión regular                       #
#                 linea  --> línea a tratar                          #
#     Return: número de ocurrencias                                  #
######################################################################
function cuenta_ocurrencias(regexp,linea)
{
	linea_aux=linea
	for (iterador=0; linea_aux ~ regexp; iterador++) {
		sub(regexp,"",linea_aux)
	}
	return iterador
}
######################################################################
#                   Nombre función: mi_getline                       #
#     Descripción: devuelve la cadena correspondiente al fichero     #
#                      y linea dados                                 #
#     Argumentos: fichero --> nombre del fichero                     #
#                 n_linea --> nº de línea                            #
#     Return: línea leída                                            #
######################################################################
function mi_getline(fichero, n_linea)
{
        cont=0
        do {
        	cont++
        	getline linea_getline < fichero
        } while (cont<n_linea)
		close(fichero)
		linea_getline=elimina_coment(linea_getline)
        return linea_getline
}

BEGIN{
	flag_R7=0
	linea_anterior=""
	
	# Expresión regular que matchea con los tipos de datos que puede ser una variable
	tipos_var="^\\s*(const)?\\s*(unsigned\\s+|signed\\s+|char\\s+|auto\\s+|bool\\s+|short\\s+|long\\s+|int\\s+|float\\s+|double\\s+)\\s*(*|&)?\\s*"
	
	# Expresión regular que matchea con los tipos de datos que puede devolver una función
	tipos_func="^\\s*(const|inline)?\\s*(unsigned\\s+|signed\\s+|char\\s+|auto\\s+|bool\\s+|short\\s+|long\\s+|int\\s+|float\\s+|double\\s+|void\\s+)\\s*(*|&)?\\s*"
	
	# Encaja con paréntesis que tengan dentro de ellos paréntesis emparejados
	parent_par="\\s*\\([^\\(\\)]*(\\([^\\(\\)]*\\)[^\\(\\)]*)*\\)"
	
	# Las reglas a 1 y 0 están en regla
	n_reglas=split(reglas,regla," ")

	# Inicializamos el array con lo errores de cada regla
	for (i in regla) {
		n_errores[i]=0
	}
	n_errores_totales=0
}{
	linea_mod=elimina_coment($0)
	
	##################################################################
	#                            REGLA 1                             #
	##################################################################
	if (regla[1] == 1) {
		if ($0 ~ /^\s*{(\s*{\s*)+$/) {
			log_awk(FILENAME";"FNR";1;0101;","D")
			n_errores[1]++
		}
		if ($0 ~ /^\s*}(\s*}\s*)+$/) {
			log_awk(FILENAME";"FNR";1;0102;","D")
			n_errores[1]++
		}
	}
	
	##################################################################
	#                            REGLA 2                             #
	##################################################################
	if (regla[2] == 1) {
		if ($0 ~ /\t/) {
			log_awk(FILENAME";"FNR";2;0201;","D")
			n_errores[2]++
		}
	}
	
	##################################################################
	#                            REGLA 3                             #
	##################################################################
	if (regla[3] == 1) {
		regexp_bucles="((if|for|while)\\s*\\(.*|}?\\s*else\\s*(if\\s*\\(.*)?|do)(\\s+|$)"
		if (linea_mod ~ "^\\s*{?\\s*"regexp_bucles && linea_mod !~ "(\\)|else\\s*|do)\\s*{") {
			n_parent=cuenta_ocurrencias("\\(",linea_mod)-cuenta_ocurrencias("\\)",linea_mod)
			if (n_parent != 0) {
				a=0
				do {
					a++
					linea_getl=mi_getline(FILENAME,FNR+a)
					n_parent+=cuenta_ocurrencias("\\(",linea_getl)-cuenta_ocurrencias("\\)",linea_getl)
					if (a >= 1000 ) {
						log_awk(" 020  Error en detección de la regla 3 para el fichero "FILENAME,"E")
					}
				} while (n_parent != 0);
				num_linea=FNR+a
				linea_bucle=linea_getl
			} else {
				num_linea=FNR
				linea_bucle=linea_mod
			}
			
			
			if (linea_bucle ~ /;\s*$/ && linea_bucle !~ /{/) {
				if (linea_bucle !~ "while\\s*"parent_par"\\s*;") {
					log_awk(FILENAME";"num_linea";3;0304;","D")
					a=1
					do {
						linea_getl=mi_getline(FILENAME,num_linea+a)
						if (linea_getl ~ /^\s*(else|while\s*\(.*\s*;)/ && linea_mod ~ /((else\s*)?if\s*\(.*|do)/) {
							log_awk(FILENAME";"num_linea+a";3;0303;","D")
							break
						} else if (linea_getl !~ /^\s*$/) {
							log_awk(FILENAME";"num_linea";3;0302;","D")
							break
						}
						a++
						if (a >= 1000 ) {
							log_awk(" 020  Error en detección de la regla 3 para el fichero "FILENAME,"E")
						}
					} while (linea_getl ~ /^\s*$/);
					n_errores[3]++
				}
			} else if (linea_bucle !~ /\)\s*{/ && linea_bucle !~ /else\s*{/ && linea_bucle !~ /do\s*{/) {
				i=1
				# Buscar corchete
				do {
					linea_getl=mi_getline(FILENAME,num_linea+i)
					if (linea_getl ~ /(^\s*{|{\s*$)/ && ( linea_getl !~ "^\\s*{?\\s*"regexp_bucles || linea_getl ~ "^\\s*{\\s*"regexp_bucles)) {
						break
					} else if (linea_mod ~ /^\s*}?\s*else\s*$/ && linea_getl ~ /^\s*if\s*\(.*/) {
						for (a=0; a < i; a++) {
							log_awk(FILENAME";"FNR+a";3;0305;","D")
						}
						log_awk(FILENAME";"FNR+i";1;0103;","D")
						n_errores[3]++
						break
					} else if (linea_getl !~ /^\s*$/) {
						j=i
						# Ver si tenemos un bucle o es una sentencia
						do {
							linea_getl=mi_getline(FILENAME,num_linea+j)
							if (linea_getl ~ /(;|})\s*$/) {
								log_awk(FILENAME";"num_linea";3;0301;","D")
								a=j+1
								do {
									linea_getl=mi_getline(FILENAME,num_linea+a)
									if (linea_getl ~ /^\s*(else|while\s*\(.*\s*;)/ && linea_mod ~ /((else\s*)?if\s*\(.*|do)/) {
										log_awk(FILENAME";"num_linea+a";3;0303;","D")
										break
									} else if (linea_getl !~ /^\s*$/) {
										log_awk(FILENAME";"num_linea+j";3;0302;","D")
										break
									}
									a++
									if (a >= 1000 ) {
										log_awk(" 020 Error en detección de la regla 3 para el fichero "FILENAME,"E")
									}
								} while (1);
								break
							} else if (linea_getl ~ /^\s*$/ || linea_getl ~ "^\\s*{?\\s*"regexp_bucles) {
								k=j
								# Buscamos fin del bucle
								do {
									linea_getl=mi_getline(FILENAME,num_linea+k)
									if (linea_getl ~ /(^\s*{|{\s*$)/) {
										l=k
										n_llaves=0
										# Buscamos fin de bucle con llave
										do {
											linea_getl=mi_getline(FILENAME,num_linea+l)
											n_llaves+=cuenta_ocurrencias("{",linea_getl)-cuenta_ocurrencias("}",linea_getl)
											if (n_llaves <= 0) {
												log_awk(FILENAME";"num_linea";3;0301;","D")
												a=l+1
												do {
													linea_getl=mi_getline(FILENAME,num_linea+a)
													if (linea_getl ~ /^\s*(else|while\s*\(.*\s*;)/ && linea_mod ~ /((else\s*)?if\s*\(.*|do)/) {
														log_awk(FILENAME";"num_linea+a";3;0303;","D")
														break
													} else if (linea_getl !~ /^\s*$/) {
														log_awk(FILENAME";"num_linea+l";3;0302;","D")
														break
													}
													a++
													if (a >= 1000 ) {
														log_awk(" 020 Error en detección de la regla 3 para el fichero "FILENAME,"E")
													}
												} while (1);
												break
											}
											l++
											if (l >= 1000 ) {
												log_awk(" 020 Error en detección de la regla 3 para el fichero "FILENAME,"E")
											}
										} while (n_llaves >= 0);
										break
									} else if (linea_getl ~ /(;|})\s*$/) {
										log_awk(FILENAME";"num_linea";3;0301;","D")
										a=k+1
										do {
											linea_getl=mi_getline(FILENAME,num_linea+a)
											if (linea_getl ~ /^\s*(else|while\s*\(.*\s*;)/ && linea_mod ~ /((else\s*)?if\s*\(.*|do)/) {
												log_awk(FILENAME";"num_linea+a";3;0303;","D")
												break
											} else if (linea_getl !~ /^\s*$/) {
												log_awk(FILENAME";"num_linea+k";3;0302;","D")
												break
											}
											a++
											if (a >= 1000 ) {
												log_awk(" 020 Error en detección de la regla 3 para el fichero "FILENAME,"E")
											}
										} while (1);
										break
									}
									k++
									if (k >= 1000 ) {
										log_awk(" 020 Error en detección de la regla 3 para el fichero "FILENAME,"E")
									}
								} while (1);
								break
							}
							j++
							if (j >= 1000 ) {
								log_awk(" 020 Error en detección de la regla 3 para el fichero "FILENAME,"E")
							}
						} while (1);
						n_errores[3]++
						break
					}
					i++
					if (i >= 1000 ) {
						log_awk(" 020 Error en detección de la regla 3 para el fichero "FILENAME,"E")
					}
				} while (1);
			}
		}
	}
	
	##################################################################
	#                            REGLA 4                             #
	##################################################################
	if (regla[4] == 1) {
		exp_reg_apert="((if|for|while|catch|switch)\\s*\\(.*|else\\s*(if\\s*\\(.*)?|do\\s*{?\\s*$|try\\s*{?\\s*$|case.*:|default:)"
		exp_reg_cierre="^\\s*}?\\s*(catch\\s*\\(.*|while\\s*\\(.*\\)\\s*;|else\\s*(if\\s*\\(.*)?)"
		
		if (linea_mod ~ "^\\s*{?}?\\s*"exp_reg_apert".*{.*}") {
			log_awk(FILENAME";"FNR";4;0405;","D")
			n_errores[4]++
		}
		
		if (linea_mod ~ "^\\s*{?}?\\s*"exp_reg_apert"(\\s+|{|$)" && linea_mod !~ exp_reg_apert" {" && linea_mod !~ "while\\s*"parent_par"\\s*;") {
			n_parent=cuenta_ocurrencias("\\(",linea_mod)-cuenta_ocurrencias("\\)",linea_mod)
			if (n_parent != 0) {
				a=0
				do {
					a++
					linea_getl=mi_getline(FILENAME,FNR+a)
					n_parent+=cuenta_ocurrencias("\\(",linea_getl)-cuenta_ocurrencias("\\)",linea_getl)
					if (a >= 1000 ) {
						log_awk(" 020 Error en detección de la regla 4 para el fichero "FILENAME,"E")
					}
				} while (n_parent != 0);
				num_linea=FNR+a
				linea_bucle=linea_getl
			} else {
				num_linea=FNR
				linea_bucle=linea_mod
			}
			
			if (linea_bucle ~ /;\s*$/ && linea_bucle ~ /(^\s*{|{\s*$)/) {
				log_awk(FILENAME";"num_linea";4;0403;","D")
				log_awk(FILENAME";"num_linea";4;0401;","D")
				n_errores[4]++
			} else {
				if (linea_bucle ~ /{\s*$/) {
					log_awk(FILENAME";"num_linea";4;0401;","D")
					n_errores[4]++
				} else {
					i=1
					do {
						linea_getl=mi_getline(FILENAME,num_linea+i)
						if (linea_getl ~ /^\s*{/) {
							log_awk(FILENAME";"num_linea";4;0401;","D")
							log_awk(FILENAME";"num_linea+i";4;0403;","D")
							n_errores[4]++
							break
						} else if (linea_getl !~ /^\s*$/) {
							break
						}
						i++
						if (i >= 1000 ) {
							log_awk(" 020 Error en detección de la regla 4 para el fichero "FILENAME,"E")
						}
					} while (linea_getl ~ /^\s*$/);
				}
			}
		}
		
		if (linea_mod ~ "^\\s*}?\\s*"exp_reg_cierre && linea_mod !~ "} "exp_reg_cierre) {
			if (linea_mod ~ /^\s*}/) {
				log_awk(FILENAME";"FNR";4;0402;","D")
				n_errores[4]++
			} else {
				i=1
				do {
					linea_getl=mi_getline(FILENAME,FNR-i)
					if (linea_getl ~ /(^\s*}|}\s*$)/) {
						j=0
						do {
							j++
							linea_getl=mi_getline(FILENAME,FNR-j)
							
							if (linea_getl ~ /(^\s*}|}\s*$)/) {
								k=j
								n_llaves=cuenta_ocurrencias("{",linea_getl)-cuenta_ocurrencias("}",linea_getl)
								do {
									k++
									if (k >= 1000 ) {
										log_awk(" 020 Error en detección de la regla 4 para el fichero "FILENAME,"E")
									}
									linea_getl=mi_getline(FILENAME,FNR-k)
									if ( n_llaves != 0) {
										n_llaves+=cuenta_ocurrencias("{",linea_getl)-cuenta_ocurrencias("}",linea_getl)
									}
									if (linea_getl ~ "^\\s*}?{\\s*"exp_reg_apert && n_llaves == 0) {
										do {
											k++
											linea_getl=mi_getline(FILENAME,FNR-k)
											if (k >= 1000 ) {
												log_awk(" 020 Error en detección de la regla 4 para el fichero "FILENAME,"E")
											}
										} while (linea_getl ~ /^\s*$/);
									}		
								} while (linea_getl !~ "^\\s*{?}?\\s*"exp_reg_apert || n_llaves != 0);
								if (linea_getl ~ /((else\s*)?if\s*\(.*|do|try)/) {
									log_awk(FILENAME";"FNR";4;0402;","D")
									log_awk(FILENAME";"FNR-i";4;0404;","D")
									n_errores[4]++
									break
								} else if (linea_getl !~ "^\\s*{?}?\\s*"exp_reg_apert) {
									break
								}
							}
							
							if (j >= 1000 ) {
								log_awk(" 020 Error en detección de la regla 4 para el fichero "FILENAME,"E")
							}
						} while (n_llaves >= 0);
						
						break
					} else if (linea_getl !~ /^\s*$/) {
						break
					}
					i++
					if (i >= 1000 ) {
						log_awk(" 020 Error en detección de la regla 4 para el fichero "FILENAME,"E")
					}
				} while (1);
			}
		}
	}

	##################################################################
	#                            REGLA 5                             #
	##################################################################
	if (regla[5] == 1) {
		if ( (linea_mod ~ /\s*(class|namespace|struct|enum)\s+/ || linea_mod ~ tipos_func"[^\\(]+\\(.*\\)") && linea_mod ~ /{\s*$/) {
			log_awk(FILENAME";"FNR";5;0501;","D")
			n_errores[5]++
		}
	}
	
	##################################################################
	#                            REGLA 6                             #
	##################################################################
	if (regla[6] == 1) {
		if (linea_mod ~ tipos_var"[^,;=\\(]+(=[^,;=]+)?,([^,;=\\(]+(=[^,;=]+)?,?)+\\s*;\\s*$" && linea_mod !~ /\(/ && linea_mod !~ /\)/ && $0 !~ /\/\/.*,/) {
			linea_aux=linea_mod
			sub(/{.*}/,"{}", linea_aux)
			if (linea_aux ~ tipos_var"[^,;=\\(]+(=[^,;=]+)?,([^,;=\\(]+(=[^,;=]+)?,?)+\\s*;\\s*$") {
				log_awk(FILENAME";"FNR";6;0601;","D")
				n_errores[6]++
			}
		} else if (linea_mod ~ tipos_var"([^,;=\\(]+(=[^,;=]+)?,)+[^,;=\\(]+(=[^,;=]+)?(,|[^,;=\\)])\\s*$" && linea_mod !~ /\(/ && linea_mod !~ /\)/ && $0 !~ /\/\/.*,/) {
			linea_aux=linea_mod
			flag_R6=cuenta_ocurrencias("{",linea_mod)-cuenta_ocurrencias("}",linea_mod)
			if (flag_R6=0) {
				sub(/{.*}/,"{}", linea_aux)
			} else {
				sub(/{.*/,"{", linea_aux)
			}
			if (linea_aux ~ tipos_var"([^,;=\\(]+(=[^,;=]+)?,)+[^,;=\\(]+(=[^,;=]+)?,?\\s*$") {
				log_awk(FILENAME";"FNR";6;0601;","D")
				i=1
				do {
					linea_getl=mi_getline(FILENAME,FNR+i)
					if (flag_R6 != 0) {
						flag_R6+=cuenta_ocurrencias("{",linea_getl)-cuenta_ocurrencias("}",linea_getl)
						if (flag_R6 == 0) {
							sub(/.*}/,"}", linea_getl)
						} else {
							sub(/.*/,"", linea_getl)
						}
					}
					
					if (linea_getl !~ /^\s*,/) {
						log_awk(FILENAME";"FNR+i";6;0603;","D")
					}
					
					log_awk(FILENAME";"FNR+i";6;0602;","D")
					
					if (linea_getl !~ /;\s*$/ && linea_getl !~ /,\s*$/) {
						log_awk(FILENAME";"FNR+i";6;0604;","D")
					}
					
					if (linea_getl ~ /;\s*$/) {
						break
					}
					i++
					if (i >= 1000 ) {
						log_awk(" 020 Error en detección de la regla 6 para el fichero "FILENAME,"E")
					}
				} while (linea_getl !~ /;\s*$/);
				n_errores[6]++
			}
		}
	}
	
	##################################################################
	#                            REGLA 7                             #
	##################################################################
	if (regla[7] == 1) {
		if ($0 ~ /\/\/+[^\/]+/ && $0 !~ /\/\/+ [^\s]+/) {
			log_awk(FILENAME";"FNR";7;0701;","D")
			n_errores[7]++
		}
		if ($0 ~ /([^\/]|^)\/\*.*/ && $0 !~ /\*\//) {
			flag_R7=1
			if ($0 ~ /\/(\*)+\s*$/) {
				log_awk(FILENAME";"FNR";7;0702;","D")
			} else {
				log_awk(FILENAME";"FNR";7;0703;","D")
			}
			n_errores[7]++
		} else if ($0 ~ /.*\*\// && $0 !~ /\/\*/) {
			flag_R7=0
			if ($0 ~ /^\s*(\*)+\//) {
				log_awk(FILENAME";"FNR";7;0704;","D")
			} else {
				log_awk(FILENAME";"FNR";7;0705;","D")
			}
		} else if (flag_R7) {
			log_awk(FILENAME";"FNR";7;0706;","D")
		}
	}
	
	##################################################################
	#                            REGLA 8                             #
	##################################################################
	if (regla[8] == 1) {
		if (linea_mod ~ /^.*((\|\||&&).*)+$/ && linea_mod !~ /^.*(\s(\|\||&&)\s.*)+$/) {
			log_awk(FILENAME";"FNR";8;0801;","D")
			n_errores[8]++
		}
	}
		
	##################################################################
	#                        REGLA REPORTE 1                         #
	##################################################################
	if (regla[9] == 1) {
		if (linea_mod ~ tipos_var"([^,;=\\(]+(=[^,;=]+)?,?)+\\s*;\\s*$" && linea_mod !~ tipos_var"([^,;=\\(]+=[^,;=]+,?)+\\s*;\\s*$" && linea_mod !~ /\(/ && linea_mod !~ /\)/ && $0 !~ /\/\/.*,/) {
			linea_aux=linea_mod
			sub(/{.*}/,"{}", linea_aux)
			if (linea_aux ~ tipos_var"([^,;=\\(]+(=[^,;=]+)?,?)+\\s*;\\s*$" && linea_aux !~ tipos_var"([^,;=\\(]+=[^,;=]+,?)+\\s*;\\s*$") {
				log_awk(FILENAME";"FNR";9;1;","R")
				n_errores[9]++
			}
		} else if (linea_mod ~ tipos_var"([^,;=\\(]+(=[^,;=]+)?,)+[^,;=\\(]+(=[^,;=]+)?,?\\s*$" && linea_mod !~ /\(/ && linea_mod !~ /\)/ && $0 !~ /\/\/.*,/) {
			linea_aux=linea_mod
			flag_R8=cuenta_ocurrencias("{",linea_mod)-cuenta_ocurrencias("}",linea_mod)
			if (flag_R8=0) {
				sub(/{.*}/,"{}", linea_aux)
			} else {
				sub(/{.*/,"{", linea_aux)
			}
			if (linea_aux ~ tipos_var"([^,;=\\(]+(=[^,;=]+)?,)+[^,;=\\(]+(=[^,;=]+)?,?\\s*$") {
				if (linea_aux !~ tipos_var"([^,;=\\(]+=[^,;=]+,)+[^,;=\\(]+=[^,;=]+,?\\s*$") {
					log_awk(FILENAME";"FNR";9;1;","R")
					n_errores[9]++
				}
				i=1
				do {
					linea_getl=mi_getline(FILENAME,FNR+i)
					if (flag_R8 != 0) {
						flag_R8+=cuenta_ocurrencias("{",linea_getl)-cuenta_ocurrencias("}",linea_getl)
						if (flag_R8 == 0) {
							sub(/.*}/,"}", linea_getl)
						} else {
							sub(/.*/,"", linea_getl)
						}
					}
					if (linea_getl !~ /^\s*,?([^,;=]+=[^,;=]+,)*[^,;=]+=[^,;=]+,?\s*$/) {
						log_awk(FILENAME";"FNR+i";9;1;","R")
						n_errores[9]++
					}
					i++
					if (i >= 1000 ) {
						log_awk(" 020 Error en detección de la regla reporte 1 para el fichero "FILENAME,"E")
					}
				} while (linea_getl !~ /;\s*$/);
			}
		}
	}
	
	##################################################################
	#                        REGLA REPORTE 2                         #
	##################################################################
	if (regla[10] == 1) {
		if (length($0) > long_max) {
			log_awk(FILENAME";"FNR";10;2;","R")
			n_errores[10]++
		}
	}
	
	##################################################################
	#                        REGLA REPORTE 3                         #
	##################################################################
	if (regla[11] == 1) {
		if (linea_mod ~ tipos_func"[^\\(]+\\(.*\\)") {
			if (linea_mod ~ /\)\s*(const)?\s*{\s*$/) {
				if (linea_anterior !~ /(\/\/|\*\/)/) {
					log_awk(FILENAME";"FNR";11;3;","R")
					n_errores[11]++
				}
			}
		}
	}
	
	
	if ($0 !~ /^\s*$/) {
		linea_anterior=$0
	}
}
END {
	log_awk("Estadísticas de detección","TI")
	for (i=1; i <= n_reglas; i++) {
		if (regla[i] == 1) {
			if (i <= n_reglas-num_reg_rep) {
				log_awk("Nº errores regla " i ": " n_errores[i],"TI")
			} else {
				log_awk("Nº reportes regla " i ": " n_errores[i],"TI")
			}
			n_errores_totales+=n_errores[i]
		}
	}
	log_awk("Nº errores totales: " n_errores_totales,"TI")
	
}' $WORKSHOP

err_detector=$?
if [[ $err_detector == 0 ]]
then
	log "DETECTOR FINALIZÓ SIN INCIDENCIAS" "TI"
else
	log "016 DETECTOR NO FINALIZÓ CORRECTAMENTE" "E"
fi

