######################################################################
#                         Inicio de lib_TC.sh                        #
######################################################################
# CONSTANTES
export CONFIG="${MI_PATH}/config"
export CFG_AJUSTES="${CONFIG}/ajustes_TC.cfg"
export CFG_REGLAS="${CONFIG}/reglas_TC.cfg"
export CORRECTOR="${MI_PATH}/scripts/corrector_TC.sh"
export DETECTOR="${MI_PATH}/scripts/detector_TC.sh"
export SALIDA="${MI_PATH}/salida"
export ENTRADA="${MI_PATH}/entrada"

# VARIABLES
export REPORTE="reporte_TC"
export INFORME_DET="informe_detector_TC"
export RUTA_FICHEROS=""
export SOBRESCRIBIR=""
export LONG_MAX=""
export WORKSHOP=""
export SECUENCIA=""
export regla

######################################################################
#                   Nombre función: log                              #
#     Descripción: imprime la cadena que recibe en el fichero        #
#       correspondiente según el nivel del texto a imprimir          #
#     Argumentos: $1 --> cadena a imprimir                           #
#                 $2 --> Nivel del texto                             #
#                      I --> Información en log                      #
#                      TI -> Información en terminal e log           #
#                      E --> Error                                   #
######################################################################
function log()
{
	case $2 in
		"I")
			# Pinta en log_TC.txt
			echo "[INFO] ----" $1 >> $LOG
			;;
		"TI")
			# Pinta en log_TC.txt y en el terminal
			echo "[INFO] ----" $1 | tee -a $LOG
			;;
		"E")
			# Pinta en log_TC.txt y en el terminal
			echo "[ERROR]" $1 | tee -a $LOG
			exit 1;
			;;
		*)
			# Error en el parámetro
			echo "[ERROR] 000 Parámetro de nivel incorrecto" | tee -a $LOG
			exit 1;
			;;
	esac
}

######################################################################
#                   Nombre función: ayuda_lanz                       #
#     Descripción: muestra los posibles parámetros de ejecución      #
######################################################################
function ayuda_lanz()
{
	log "Parámetros de ejecución admitidos:" "I"
	echo "                D --> Solo detección"
	echo "                DC --> Detección y corrección"
	echo "                R --> Solo reportes de warnings"
	echo "                --help --> Ayuda"
}

######################################################################
#                   Nombre función: check_dir                        #
#       Descripción: comprueba la existencia de directorios          #
######################################################################
function check_dir()
{
	if [ -d $ENTRADA ]
	then
		log "Existe directorio de entrada" "I"
	else
		log "Directorio de entrada creado" "TI"
		mkdir $ENTRADA
	fi

	if [ -d $SALIDA ]
	then
		log "Existe directorio de salida" "I"
	else
		log "Directorio de salida creado" "TI"
		mkdir $SALIDA
	fi

	if [ -d $CONFIG ]
	then
		log "Existe directorio de configuración" "I"
		if [ -f $CFG_AJUSTES ]
		then
			log "ajustes_TC.cfg existe" "I"
		else
			log "004 ajustes_TC.cfg no existe" "E"
		fi
		if [ -f $CFG_REGLAS ]
		then
			log "reglas_TC.cfg existe" "I"
		else
			log "005 reglas_TC.cfg no existe" "E"
		fi
	else
		log "003 Directorio config no encontrado" "E"
	fi
}

######################################################################
#                   Nombre función: check_conf                       #
#  Descripción: extrae las variables necesarias del fichero de conf  #
######################################################################
function check_conf()
{
	. $CFG_AJUSTES
	if [ -z $RUTA_FICHEROS ]
	then
		log "006 Ruta de ficheros a modificar no especificada" "E"
	else
		if [ -d $RUTA_FICHEROS ]
		then
			log "Ruta de ficheros a modificar: ${RUTA_FICHEROS}" "TI"
			if [[ $MI_PATH != $RUTA_FICHEROS ]]
			then
				WORKSHOP=$(find ${RUTA_FICHEROS} -iname "*.c" -or -iname "*.cpp" -or -iname "*.h" -or -iname "*.hpp") #MOD_REQ#
				if [[ -z $WORKSHOP ]]
				then
					log "009 No se encontraron ficheros que modificar" "E"
				else
					n_fich=$(find ${RUTA_FICHEROS} -iname "*.c" -or -iname "*.cpp" -or -iname "*.h" -or -iname "*.hpp" | wc -l) #MOD_REQ#
					log "${n_fich} fichero(s) a modificar encontrado(s)" "TI"
				fi
			else
				log "008 La ruta de los ficheros a modificar no es válida" "E"
			fi
		else
			log "007 Ruta de ficheros a modificar no existe" "E"
		fi
	fi
	
	if [[ $SOBRESCRIBIR == "SI" ]]
	then
		log "Sobreescritura de archivos activada" "TI"
		log "¿Seguro que quiere SOBRESCRIBIR los archivos?" "TI"
		log "Escriba NO para abandonar o cualquier otra cadena para continuar" "TI"
		read respuesta
		log "Escribió: ${respuesta}" "I"
		if [[ $respuesta == "NO" ]]
		then
			log "Operación cancelada, abandonando herramienta" "TI"
			exit 2
		fi
	elif [[ $SOBRESCRIBIR == "NO" ]]
	then
		log "Sobreescritura de archivos desactivada" "TI"
	else
		log "010 Variable SOBRESCRIBIR debe ser SI o NO" "E"
	fi
	
	if [[ $LONG_MAX =~ ^[0-9]+$ ]]
	then
		log "Patrón de longitud máxima OK" "I"
	else
		log "019 Variable LONG_MAX debe de ser un número" "E"
	fi
	
	if [[ $SECUENCIA =~ ^[0-9]+$ ]]
	then
		log "Patrón de secuencia OK" "I"
		SECUENCIA=$(($SECUENCIA + 1))
		log "Número de secuencia actualizado a ${SECUENCIA}" "TI"
		rm $CFG_AJUSTES
		echo "RUTA_FICHEROS=\""$RUTA_FICHEROS"\"" >> $CFG_AJUSTES
		echo "SOBRESCRIBIR="$SOBRESCRIBIR >> $CFG_AJUSTES
		echo "LONG_MAX="$LONG_MAX >> $CFG_AJUSTES
		echo "SECUENCIA="$SECUENCIA >> $CFG_AJUSTES
	else
		log "011 Variable SECUENCIA debe de ser un número" "E"
	fi
	
}

######################################################################
#                   Nombre función: check_perm                       #
#     Descripción: comprueba que los scripts tienen permisos         #
######################################################################
function check_perm()
{
	if [ -f $DETECTOR ]
	then
		log "detector_TC.sh existe" "I"
		if [ -x $DETECTOR ]
		then
			log "detector_TC.sh tiene permisos de ejecución" "I"
		else
			chmod +x $DETECTOR
			log "Permisos otorgados a detector_TC.sh" "TI"
		fi
	else
		log "012 detector_TC.sh no existe" "E"
	fi
		
	if [ -f $CORRECTOR ]
	then
		log "corrector_TC.sh existe" "I"
		if [ -x $CORRECTOR ]
		then
			log "corrector_TC.sh tiene permisos de ejecución" "I"
		else
			chmod +x $CORRECTOR
			log "Permisos otorgados a corrector_TC.sh" "TI"
		fi
	else
		log "013 corrector_TC.sh no existe" "E"
	fi
}



######################################################################
#                   Nombre función: check_param                      #
#   Descripción: comprueba que los parámetros del lanzador son OK    #
#     Argumentos: $1 --> $1 de ejecución                             #
#                 $2 --> $# número de parámetros de ejecución        #
######################################################################
function check_param()
{
	if [[ $2 == "1" ]]
	then
		log "Número de parámetros de ejecución correctos" "I"
		case $1 in
		"D")
			log "Lanzador en modo detección" "TI"
			;;
		"DC")
			log "Lanzador en modo detección y corrección" "TI"
			;;
		"R")
			log "Lanzador en modo reportes " "TI"
			;;
		"--help")
			log "Ayuda solicitada en ejecución" "TI"
			ayuda_lanz
			exit 1;
			;;
		*)
			log "014 Parámetros de ejecución incorrectos, consulte la ayuda con --help" "E"
			;;
		esac
	else 
		log "014 Parámetros de ejecución incorrectos, consulte la ayuda con --help" "E"
	fi
}

######################################################################
#                   Nombre función: check_reglas                     #
#  Descripción: extrae las variables necesarias del fichero de conf  #
######################################################################
function check_reglas()
{
	i=0
	for r in $(cat ${CFG_REGLAS})
	do
		i=$(($i + 1))
		if [[ $r =~ R[0-9]+\=NO$ ]]
		then
			regla[$i]=0
		elif [[ $r =~ R[0-9]+\=SI$ ]]
		then
			regla[$i]=1
			log "Regla ${i} activa" "TI"
		else
			log "015 Error en reglas_TC.cfg: las reglas deben estar a SI o NO" "E"
		fi
	done	
}

######################################################################
#                   Nombre función: crea_dir_seq                     #
#  Descripción: crea el directorio con la secuencia correspondiente  #
######################################################################
function crea_dir_seq()
{
	# Directorio de la salida para esta secuencia
	export SAL_SEQ="${SALIDA}/${SECUENCIA}"

	# Sobreescribimos en caso de que ya haya una carpeta con la secuencia actual
	[ -d $SAL_SEQ ] && rm -rf $SAL_SEQ && log "Directorio de salida sobreescrito (nº de secuencia repetido)" "TI"

	# Creamos el directorio donde se guardarán los archivos
	mkdir $SAL_SEQ

	# Movemos el log a su nueva ubicación
	[ -f $LOG ] && mv $LOG "${SAL_SEQ}/${LOG}_${SECUENCIA}.txt"

	# Incluímos el PATH en el nombre de los archivos
	LOG="${SAL_SEQ}/${LOG}_${SECUENCIA}.txt"
	INFORME_DET="${SAL_SEQ}/${INFORME_DET}_${SECUENCIA}.csv"
	REPORTE="${SAL_SEQ}/${REPORTE}_${SECUENCIA}.csv"
}

######################################################################
#                   Nombre función: crea_dir_correg                  #
#  Descripción: crea los directorios de la ruta de ficheros escogida #
######################################################################
function crea_dir_correg
{
	if [[ $SOBRESCRIBIR == "SI" ]]
	then
		RUTA_CORREGIDOS=$RUTA_FICHEROS"_copy"
		mkdir $RUTA_CORREGIDOS
		log "Directorio ${RUTA_CORREGIDOS} creado" "I"
		
		for i in $WORKSHOP
		do
			fich="${i#${RUTA_FICHEROS}}"
			dir="${fich%/*}"
			
			mkdir -p "${RUTA_CORREGIDOS}/${dir}"
		done
	elif [[ $SOBRESCRIBIR == "NO" ]]
	then
		export CORREGIDOS="${SAL_SEQ}/corregidos"
		mkdir $CORREGIDOS
		log "Directorio ${CORREGIDOS} creado" "I"
		export ORIGINALES="${SAL_SEQ}/originales"
		mkdir  $ORIGINALES
		log "Directorio ${ORIGINALES} creado" "I"
		
		for i in $WORKSHOP
		do
			fich="${i#${RUTA_FICHEROS}}"
			dir="${fich%/*}"
			
			mkdir -p "${CORREGIDOS}/${dir}"
			mkdir -p "${ORIGINALES}/${dir}"
			cp -ra $i "${ORIGINALES}${fich}" 
		done

		RUTA_CORREGIDOS=$CORREGIDOS
	fi
}

######################################################################
#                   Nombre función: check_informe_det                #
#  Descripción: comprueba que el informe del detector trae al menos  #
#                   1 error                                          #
######################################################################
function check_informe_det
{
	if [[ $(cat ${INFORME_DET}) =~ [0-9] ]]
	then
		log "Los ficheros contienen errores" "TI"
		return 1
	elif (( regla[1] == 1 ))
	then
		log "Los ficheros no contienen errores, R1 activa" "TI"
		return 1
	else
		log "Los ficheros no contienen errores" "TI"
		return 0
	fi
	
}

# FUNCIONES
export -f log
export -f ayuda_lanz
export -f check_dir
export -f check_conf
export -f check_perm
export -f check_param
export -f check_reglas
export -f crea_dir_seq
export -f crea_dir_correg
export -f check_informe_det